package org.expeditee.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowStateListener;

import javax.swing.JFrame;

import org.expeditee.AbsoluteLayout;
import org.expeditee.actions.Actions;
import org.expeditee.actions.Misc;
import org.expeditee.io.Logger;

/**
 * The Main GUI class, comprises what people will see on the screen.<br>
 * Note: Each Object (Item) is responsible for drawing itself on the screen.<br>
 * Note2: The Frame is registered as a MouseListener and KeyListener, and
 * processes any Events.
 * 
 * @author jdm18
 * 
 */
public class Browser extends JFrame implements ComponentListener,
		WindowListener, WindowStateListener {
	
	/**
	 * Default version - just to stop eclipse from complaining about it.
	 */
	private static final long serialVersionUID = 1L;

	// private static final JScrollPane scrollPane = new JScrollPane();

	public static Browser _theBrowser;
	
	/**
	 * Constructs a new Browser object, then launches it
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		_theBrowser = new Browser();
		// Why do we want to ignore repaint?
		// b.setIgnoreRepaint(true);
		_theBrowser.requestFocus();
		// FrameGraphics.ForceRepaint();
	}
	
	public void setSizes(Dimension size) {
		setSize(size);
		setPreferredSize(size);
		addWindowListener(this);
		addWindowStateListener(this);

		// center the frame on the screen
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		double xpos = screen.getWidth() / 2;
		double ypos = screen.getHeight() / 2;
		setLocation((int) (xpos - (size.getWidth() / 2)), (int) (ypos - (size
				.getHeight() / 2)));

		FrameGraphics.setMaxSize(this.getContentPane().getSize());
	}

	public Browser() {
		// Use the default values initially so we can load the profile frame
		setSizes(new Dimension(UserSettings.InitialWidth,
				UserSettings.InitialHeight));

		UserSettings.Init();
		UserSettings.Username = System.getProperty("user.name");
		String userName = FrameIO.ConvertToValidFramesetName(UserSettings.Username);
		Frame profile = FrameIO.LoadProfile(userName);
		if (profile == null) {
			try {
				profile = FrameIO.CreateNewProfile(userName);
			} catch (Exception e) {
				//TODO tell the user that there was a problem creating the profile frame and close nicely
				assert(false);
			}
		}
		FrameUtils.ParseProfile(profile);

		// Now reset the size of the windows to the size specified in the users
		// profile
		setSizes(new Dimension(UserSettings.InitialWidth,
				UserSettings.InitialHeight));

		//set the layout to absolute layout for widgets
		this.getContentPane().setLayout(new AbsoluteLayout());
		//enable the glasspane-for capturing all mouse events
		this.setGlassPane(new MouseEventRouter(getJMenuBar(), getContentPane()));
		this.getGlassPane().setVisible(true);
		this.getContentPane().setBackground(Color.white);
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		addComponentListener(this);
		pack();
		try {
			go();
		} catch (Exception e) {
			e.printStackTrace();
			Logger.Log(e);
		}
		// FrameGraphics.ForceRepaint();
	}

	Graphics2D g;

	/**
	 * Loads in the Menus and the first Frame.
	 */
	public void go() {

		Actions.Init();
		DisplayIO.Init(this);
		// Set visible must be just after DisplayIO.Init for the message box to
		// be the right size
		setVisible(true);

		setupGraphics();

		// required to accept TAB key
		setFocusTraversalKeysEnabled(false);

		assert (UserSettings.FirstFrame != null);
		Frame firstFrame = FrameIO.LoadFrame(UserSettings.FirstFrame);

		DisplayIO.setCurrentFrame(firstFrame);

		FrameKeyboardActions keyboardListner = new FrameKeyboardActions();
		this.getContentPane().addKeyListener(keyboardListner);
		this.addKeyListener(keyboardListner);
	}

	private void setupGraphics() {
		g = (Graphics2D) this.getContentPane().getGraphics();
		assert g != null;
		g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
				RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
		g.setFont(g.getFont().deriveFont(40f));
		FrameGraphics.setDisplayGraphics(g);
	}

	@Override
	public void paint(Graphics g) {
		super.paint(g);
		FrameGraphics.Repaint();
	}

	/**
	 * @inheritDoc
	 */
	public void componentResized(ComponentEvent e) {
		setSizes(this.getSize());
		setupGraphics();
		repaint();
	}

	/**
	 * @inheritDoc
	 */
	public void componentMoved(ComponentEvent e) {
		// FrameGraphics.setMaxSize(this.getSize());
	}

	/**
	 * @inheritDoc
	 */
	public void componentShown(ComponentEvent e) {
	}

	/**
	 * @inheritDoc
	 */
	public void componentHidden(ComponentEvent e) {
	}

	public void windowClosing(WindowEvent e) {
		Misc.Exit();
	}

	public void windowClosed(WindowEvent e) {
	}

	public void windowOpened(WindowEvent e) {
	}

	public void windowIconified(WindowEvent e) {
	}

	public void windowDeiconified(WindowEvent e) {
	}

	public void windowActivated(WindowEvent e) {
		// System.out.println("Activated");
	}

	public void windowDeactivated(WindowEvent e) {
	}

	/*
	 * public void windowGainedFocus(WindowEvent e) { FrameGraphics.Repaint(); }
	 * 
	 * public void windowLostFocus(WindowEvent e) { //FrameGraphics.Repaint(); }
	 */
	public void windowStateChanged(WindowEvent e) {
		//FrameGraphics.Repaint();
		//System.out.println('C');
	}

	public int getDrawingAreaX() {
		// return scrollPane.getLocationOnScreen().x;
		return this.getLocationOnScreen().x;
	}

	public int getDrawingAreaY() {
		// return scrollPane.getLocationOnScreen().y;
		return this.getLocationOnScreen().y;
	}
}
